@@ -6,14 +6,11 @@
 
 #include <assert.h>
 #include <stdio.h>
-#include <string.h>
 
 #include <algorithm>
+#include <string>
 
-#include "vpl/mfxdispatcher.h"
-#include "vpl/mfxjpeg.h"
-#include "vpl/mfxstructures.h"
-#include "vpl/mfxvp8.h"
+#include "vpl/mfx.h"
 
 #define DECODE_FOURCC(ch) ch & 0xff, ch >> 8 & 0xff, ch >> 16 & 0xff, ch >> 24 & 0xff
 
@@ -96,6 +93,17 @@ const char *_print_MediaAdapterType(mfxMediaAdapterType type) {
     return "<unknown media adapter type>";
 }
 
+const char *_print_EncodeStatsType(mfxU16 type) {
+    switch (type) {
+        STRING_OPTION(MFX_ENCODESTATS_LEVEL_BLK);
+        STRING_OPTION(MFX_ENCODESTATS_LEVEL_SLICE);
+        STRING_OPTION(MFX_ENCODESTATS_LEVEL_TILE);
+        STRING_OPTION(MFX_ENCODESTATS_LEVEL_FRAME);
+    }
+
+    return "<unknown encode stats type>";
+}
+
 const char *_print_ResourceType(mfxResourceType type) {
     switch (type) {
         STRING_OPTION(MFX_RESOURCE_SYSTEM_SURFACE);
@@ -232,13 +240,53 @@ int main(int argc, char *argv[]) {
 
     bool bPrintImplementedFunctions = false;
     bool bFullInfo                  = true;
-    if (argc == 2) {
-        if (!strncmp(argv[1], "-f", 2)) {
+    bool bPrintExtendedDeviceID     = false;
+    bool bRequireD3D9               = false;
+
+    for (int argIdx = 1; argIdx < argc; argIdx++) {
+        std::string nextArg(argv[argIdx]);
+
+        if (nextArg == "-f") {
             bPrintImplementedFunctions = true;
         }
-        else if (!strncmp(argv[1], "-b", 2)) {
+#ifdef ONEVPL_EXPERIMENTAL
+        else if (nextArg == "-ex") {
+            bPrintExtendedDeviceID = true;
+        }
+#endif
+        else if (nextArg == "-b") {
             bFullInfo = false;
         }
+        else if (nextArg == "-d3d9") {
+            bRequireD3D9 = true;
+        }
+        else {
+            printf("Error - unknown option %s\n", nextArg.c_str());
+            return -1;
+        }
+    }
+
+    if (bRequireD3D9) {
+        printf("Warning - Enumerating D3D9 implementations ONLY\n");
+        mfxConfig cfg = MFXCreateConfig(loader);
+        if (!cfg) {
+            printf("Error - MFXCreateConfig() returned null\n");
+            return -1;
+        }
+
+        mfxVariant var      = {};
+        var.Version.Version = MFX_VARIANT_VERSION;
+        var.Type            = MFX_VARIANT_TYPE_U32;
+        var.Data.U32        = MFX_ACCEL_MODE_VIA_D3D9;
+
+        mfxStatus sts =
+            MFXSetConfigFilterProperty(cfg,
+                                       (const mfxU8 *)"mfxImplDescription.AccelerationMode",
+                                       var);
+        if (sts) {
+            printf("Error - MFXSetConfigFilterProperty() returned %d\n", sts);
+            return -1;
+        }
     }
 
     int i = 0;
@@ -372,6 +420,29 @@ int main(int argc, char *argv[]) {
                 printf("%4sBiDirectionalPrediction: %hu\n",
                        "",
                        enc->Codecs[codec].BiDirectionalPrediction);
+
+#ifdef ONEVPL_EXPERIMENTAL
+                // Once ReportedStats is moved out of experimental API the struct version of mfxEncoderDescription should
+                //   be updated, and that can be used to know whether this field is valid.
+                // For now, just check implementation API version.
+                mfxVersion reqApiVersionReportedStats = {};
+                reqApiVersionReportedStats.Major      = 2;
+                reqApiVersionReportedStats.Minor      = 7;
+                if (idesc->ApiVersion.Version >= reqApiVersionReportedStats.Version) {
+                    mfxU16 reportedStats = enc->Codecs[codec].ReportedStats;
+                    if (reportedStats) {
+                        for (mfxU16 statMask = 1; statMask != 0; statMask <<= 1) {
+                            if (reportedStats & statMask) {
+                                const char *statStr = _print_EncodeStatsType(statMask);
+                                printf("%4sReportedStats: %s\n", "", statStr);
+                            }
+                        }
+                    }
+                    else {
+                        printf("%4sReportedStats: 0\n", "");
+                    }
+                }
+#endif
                 for (int profile = 0; profile < enc->Codecs[codec].NumProfiles; profile++) {
                     printf("%6sProfile: %s\n",
                            "",
@@ -509,6 +580,48 @@ int main(int argc, char *argv[]) {
             }
         }
 
+#ifdef ONEVPL_EXPERIMENTAL
+        if (bPrintExtendedDeviceID) {
+            mfxExtendedDeviceId *idescDevice;
+
+            mfxStatus sts = MFXEnumImplementations(loader,
+                                                   i,
+                                                   MFX_IMPLCAPS_DEVICE_ID_EXTENDED,
+                                                   reinterpret_cast<mfxHDL *>(&idescDevice));
+            if (sts == MFX_ERR_NONE) {
+                printf("%2sExtended DeviceID's:\n", "");
+                printf("%6sVendorID: 0x%04X\n", "", idescDevice->VendorID);
+                printf("%6sDeviceID: 0x%04X\n", "", idescDevice->DeviceID);
+
+                printf("%6sPCIDomain: 0x%08X\n", "", idescDevice->PCIDomain);
+                printf("%6sPCIBus: 0x%08X\n", "", idescDevice->PCIBus);
+                printf("%6sPCIdevice: 0x%08X\n", "", idescDevice->PCIDevice);
+                printf("%6sPCIFunction: 0x%08X\n", "", idescDevice->PCIFunction);
+
+                if (idescDevice->LUIDValid) {
+                    printf("%6sDeviceLUID: ", "");
+                    for (mfxU32 idx = 0; idx < 8; idx++) {
+                        printf("%02x", idescDevice->DeviceLUID[7 - idx]);
+                    }
+                    printf("\n");
+                    printf("%6sLUIDDeviceNodeMask: 0x%04X\n", "", idescDevice->LUIDDeviceNodeMask);
+                }
+
+                printf("%6sLUIDValid: 0x%04X\n", "", idescDevice->LUIDValid);
+
+                printf("%6sDRMRenderNodeNum: %d\n", "", idescDevice->DRMRenderNodeNum);
+                printf("%6sDRMPrimaryNodeNum: 0x%04X\n", "", idescDevice->DRMPrimaryNodeNum);
+
+                printf("%6sDeviceName: %s\n", "", idescDevice->DeviceName);
+                MFXDispReleaseImplDescription(loader, idescDevice);
+            }
+
+            else {
+                printf("%2sWarning - MFX_IMPLCAPS_DEVICE_ID_EXTENDED not supported\n", "");
+            }
+        }
+#endif
+
         i++;
     }
 
