@@ -114,12 +114,12 @@ mfxStatus CSmplYUVReader::Init(std::list<msdk_string> inputs,
 
     if (MFX_FOURCC_NV12 != ColorFormat && MFX_FOURCC_YV12 != ColorFormat &&
         MFX_FOURCC_I420 != ColorFormat && MFX_FOURCC_YUY2 != ColorFormat &&
-        MFX_FOURCC_RGB4 != ColorFormat && MFX_FOURCC_BGR4 != ColorFormat &&
-        MFX_FOURCC_P010 != ColorFormat && MFX_FOURCC_P210 != ColorFormat &&
-        MFX_FOURCC_AYUV != ColorFormat && MFX_FOURCC_A2RGB10 != ColorFormat &&
-        MFX_FOURCC_Y210 != ColorFormat && MFX_FOURCC_Y410 != ColorFormat &&
-        MFX_FOURCC_P016 != ColorFormat && MFX_FOURCC_Y216 != ColorFormat &&
-        MFX_FOURCC_I010 != ColorFormat) {
+        MFX_FOURCC_UYVY != ColorFormat && MFX_FOURCC_RGB4 != ColorFormat &&
+        MFX_FOURCC_BGR4 != ColorFormat && MFX_FOURCC_P010 != ColorFormat &&
+        MFX_FOURCC_P210 != ColorFormat && MFX_FOURCC_AYUV != ColorFormat &&
+        MFX_FOURCC_A2RGB10 != ColorFormat && MFX_FOURCC_Y210 != ColorFormat &&
+        MFX_FOURCC_Y410 != ColorFormat && MFX_FOURCC_P016 != ColorFormat &&
+        MFX_FOURCC_Y216 != ColorFormat && MFX_FOURCC_I010 != ColorFormat) {
         return MFX_ERR_UNSUPPORTED;
     }
 
@@ -134,11 +134,10 @@ mfxStatus CSmplYUVReader::Init(std::list<msdk_string> inputs,
     }
 
     for (ls_iterator it = inputs.begin(); it != inputs.end(); it++) {
-        FILE* f = 0;
+        m_files.push_back(NULL);
+        auto& f = m_files.back();
         MSDK_FOPEN(f, (*it).c_str(), MSDK_STRING("rb"));
         MSDK_CHECK_POINTER(f, MFX_ERR_NULL_PTR);
-
-        m_files.push_back(f);
     }
 
     m_ColorFormat = ColorFormat;
@@ -246,8 +245,8 @@ mfxStatus CSmplYUVReader::LoadNextFrame(mfxFrameSurface1* pSurface) {
             case MFX_FOURCC_UYVY:
                 pitch = pData.Pitch;
                 ptr   = m_ColorFormat == MFX_FOURCC_YUY2
-                          ? pData.Y + pInfo.CropX * 2 + pInfo.CropY * pData.Pitch
-                          : pData.U + pInfo.CropX + pInfo.CropY * pData.Pitch;
+                            ? pData.Y + pInfo.CropX * 2 + pInfo.CropY * pData.Pitch
+                            : pData.U + pInfo.CropX + pInfo.CropY * pData.Pitch;
 
                 for (i = 0; i < h; i++) {
                     nBytesRead = (mfxU32)fread(ptr + i * pitch, 2, w, m_files[vid]);
@@ -262,8 +261,8 @@ mfxStatus CSmplYUVReader::LoadNextFrame(mfxFrameSurface1* pSurface) {
             case MFX_FOURCC_Y216:
                 pitch = pData.Pitch;
                 ptr   = ((pInfo.FourCC == MFX_FOURCC_Y210 || pInfo.FourCC == MFX_FOURCC_Y216)
-                           ? pData.Y
-                           : (mfxU8*)pData.Y410) +
+                             ? pData.Y
+                             : (mfxU8*)pData.Y410) +
                       pInfo.CropX * 4 + pInfo.CropY * pData.Pitch;
 
                 for (i = 0; i < h; i++) {
@@ -320,15 +319,10 @@ mfxStatus CSmplYUVReader::LoadNextFrame(mfxFrameSurface1* pSurface) {
             case MFX_FOURCC_YV12:
                 switch (pInfo.FourCC) {
                     case MFX_FOURCC_NV12:
-
-                        mfxU8 buf[2048]; // maximum supported chroma width for nv12
                         mfxU32 j, dstOffset[2];
                         w /= 2;
                         h /= 2;
                         ptr = pData.UV + pInfo.CropX + (pInfo.CropY / 2) * pitch;
-                        if (w > 2048) {
-                            return MFX_ERR_UNSUPPORTED;
-                        }
 
                         if (m_ColorFormat == MFX_FOURCC_I420) {
                             dstOffset[0] = 0;
@@ -340,28 +334,33 @@ mfxStatus CSmplYUVReader::LoadNextFrame(mfxFrameSurface1* pSurface) {
                         }
 
                         // load first chroma plane: U (input == I420) or V (input == YV12)
-                        for (i = 0; i < h; i++) {
-                            nBytesRead = (mfxU32)fread(buf, 1, w, m_files[vid]);
-                            if (w != nBytesRead) {
-                                return MFX_ERR_MORE_DATA;
+                        try {
+                            std::vector<mfxU8> buf(w);
+                            for (i = 0; i < h; i++) {
+                                nBytesRead = (mfxU32)fread(&buf[0], 1, w, m_files[vid]);
+                                if (w != nBytesRead) {
+                                    return MFX_ERR_MORE_DATA;
+                                }
+                                for (j = 0; j < w; j++) {
+                                    ptr[i * pitch + j * 2 + dstOffset[0]] = buf[j];
+                                }
                             }
-                            for (j = 0; j < w; j++) {
-                                ptr[i * pitch + j * 2 + dstOffset[0]] = buf[j];
+
+                            // load second chroma plane: V (input == I420) or U (input == YV12)
+                            for (i = 0; i < h; i++) {
+                                nBytesRead = (mfxU32)fread(&buf[0], 1, w, m_files[vid]);
+
+                                if (w != nBytesRead) {
+                                    return MFX_ERR_MORE_DATA;
+                                }
+                                for (j = 0; j < w; j++) {
+                                    ptr[i * pitch + j * 2 + dstOffset[1]] = buf[j];
+                                }
                             }
                         }
-
-                        // load second chroma plane: V (input == I420) or U (input == YV12)
-                        for (i = 0; i < h; i++) {
-                            nBytesRead = (mfxU32)fread(buf, 1, w, m_files[vid]);
-
-                            if (w != nBytesRead) {
-                                return MFX_ERR_MORE_DATA;
-                            }
-                            for (j = 0; j < w; j++) {
-                                ptr[i * pitch + j * 2 + dstOffset[1]] = buf[j];
-                            }
+                        catch (...) {
+                            return MFX_ERR_MEMORY_ALLOC;
                         }
-
                         break;
                     case MFX_FOURCC_YV12:
                     case MFX_FOURCC_I420:
@@ -455,6 +454,58 @@ mfxStatus CSmplYUVReader::LoadNextFrame(mfxFrameSurface1* pSurface) {
     return MFX_ERR_NONE;
 }
 
+mfxStatus CSmplYUVReader::LoadNextFrame(mfxFrameSurface1* pSurface,
+                                        int bytes_to_read,
+                                        mfxU8* buf_read) {
+    // check if reader is initialized
+    MSDK_CHECK_POINTER(pSurface, MFX_ERR_NULL_PTR);
+
+    mfxU32 vid = pSurface->Info.FrameId.ViewId;
+
+    int nBytesRead = static_cast<int>(fread(buf_read, 1, bytes_to_read, m_files[vid]));
+
+    if (bytes_to_read != nBytesRead) {
+        return MFX_ERR_MORE_DATA;
+    }
+
+    mfxU16 w, h;
+    mfxFrameInfo* pInfo = &pSurface->Info;
+    mfxFrameData* pData = &pSurface->Data;
+
+    w = pInfo->Width;
+    h = pInfo->Height;
+
+    switch (pInfo->FourCC) {
+        case MFX_FOURCC_NV12:
+            pData->Y  = buf_read;
+            pData->UV = pData->Y + w * h;
+            break;
+        case MFX_FOURCC_I420:
+            pData->Y = buf_read;
+            pData->U = pData->Y + w * h;
+            pData->V = pData->U + ((w / 2) * (h / 2));
+            break;
+
+        case MFX_FOURCC_P010:
+            pData->Y  = buf_read;
+            pData->UV = pData->Y + w * 2 * h;
+            break;
+        case MFX_FOURCC_I010:
+            pData->Y = buf_read;
+            pData->U = pData->Y + w * 2 * h;
+            pData->V = pData->U + (w * (h / 2));
+            break;
+
+        case MFX_FOURCC_RGB4:
+            pData->B = buf_read;
+            break;
+        default:
+            break;
+    }
+
+    return MFX_ERR_NONE;
+}
+
 CSmplBitstreamWriter::CSmplBitstreamWriter()
         : m_nProcessedFramesNum(0),
           m_bSkipWriting(false),
@@ -510,7 +561,7 @@ mfxStatus CSmplBitstreamWriter::WriteNextFrame(mfxBitstream* pMfxBitstream,
     MSDK_CHECK_ERROR(m_bInited, false, MFX_ERR_NOT_INITIALIZED);
     MSDK_CHECK_POINTER(pMfxBitstream, MFX_ERR_NULL_PTR);
 
-    if (isCompleteFrame) {
+    if (isCompleteFrame && pMfxBitstream->DataLength) {
         mfxU32 nBytesWritten = 0;
 
         nBytesWritten = (mfxU32)fwrite(pMfxBitstream->Data + pMfxBitstream->DataOffset,
@@ -537,6 +588,100 @@ mfxStatus CSmplBitstreamWriter::WriteNextFrame(mfxBitstream* pMfxBitstream,
     return MFX_ERR_NONE;
 }
 
+mfxStatus CSmplBitstreamWriter::WriteNextFrame(mfxBitstream*, mfxU32, mfxU32) {
+    return MFX_ERR_NOT_IMPLEMENTED;
+}
+
+mfxStatus CBitstreamWriterForParallelEncoding::WriteNextFrame(mfxBitstream* pMfxBitstream,
+                                                              mfxU32 targetID,
+                                                              mfxU32 frameNum) {
+    std::unique_lock<std::mutex> guard(m_Mutex);
+
+    mfxU32 EncoderNumber = targetID - m_BaseEncoderID;
+
+    //sanity check
+    if (m_GopSize == 0 || m_NumberOfEncoders == 0 || EncoderNumber > 64) {
+        return MFX_ERR_UNDEFINED_BEHAVIOR;
+    }
+
+    mfxU32 GlobalFrameNum = m_NumberOfEncoders * m_GopSize * ((frameNum - 1) / m_GopSize) +
+                            m_GopSize * EncoderNumber + (frameNum - 1) % m_GopSize;
+
+    if (GlobalFrameNum == (mfxU32)(m_LastWrittenFrameNumber + 1)) {
+        //write bitstream directly to file
+        m_LastWrittenFrameNumber = GlobalFrameNum;
+
+        if (pMfxBitstream->DataLength) {
+            mfxU32 nBytesWritten = 0;
+            nBytesWritten        = (mfxU32)fwrite(pMfxBitstream->Data + pMfxBitstream->DataOffset,
+                                           1,
+                                           pMfxBitstream->DataLength,
+                                           m_fSource);
+            MSDK_CHECK_NOT_EQUAL(nBytesWritten,
+                                 pMfxBitstream->DataLength,
+                                 MFX_ERR_UNDEFINED_BEHAVIOR);
+            pMfxBitstream->DataLength = 0;
+            pMfxBitstream->DataOffset = 0;
+            m_nProcessedFramesNum++;
+        }
+
+        //if it has been last frame in GOP, check can we output buffered frames from other encoders
+        for (bool bufferWritten = true; bufferWritten;) {
+            bufferWritten = false;
+            for (auto& buf : m_Buffer) {
+                if (buf.second.empty()) {
+                    continue;
+                }
+
+                if (m_FirstFrameInBuffer[buf.first] == (mfxU32)(m_LastWrittenFrameNumber + 1)) {
+                    bufferWritten        = true;
+                    mfxU32 nBytesWritten = 0;
+
+                    nBytesWritten =
+                        (mfxU32)fwrite(buf.second.data(), 1, buf.second.size(), m_fSource);
+                    if (nBytesWritten != mfxU32(buf.second.size())) {
+                        return MFX_ERR_UNDEFINED_BEHAVIOR;
+                    }
+                    buf.second.clear();
+                    m_LastWrittenFrameNumber = m_LastFrameInBuffer[buf.first];
+                }
+            }
+        }
+        return MFX_ERR_NONE;
+    }
+
+    //write bitstream to buffer
+    static const int timeToSleepInMilliseconds = 10;
+    for (int i = 0; i < MSDK_ENC_WAIT_INTERVAL / timeToSleepInMilliseconds; i++) {
+        //check if buffer holds complete GOP
+        if (m_Buffer[targetID].empty() || GlobalFrameNum == m_LastFrameInBuffer[targetID] + 1) {
+            if (m_Buffer[targetID].empty()) {
+                m_FirstFrameInBuffer[targetID] = GlobalFrameNum;
+            }
+            m_LastFrameInBuffer[targetID] = GlobalFrameNum;
+
+            m_Buffer[targetID].insert(
+                m_Buffer[targetID].end(),
+                pMfxBitstream->Data + pMfxBitstream->DataOffset,
+                pMfxBitstream->Data + pMfxBitstream->DataOffset + pMfxBitstream->DataLength);
+            return MFX_ERR_NONE;
+        }
+
+        //we have complete GOP in buffer, this channel is ahead of others, wait till buffer will be written to file
+        guard.unlock();
+        MSDK_SLEEP(timeToSleepInMilliseconds);
+        guard.lock();
+    }
+
+    //we've failed to write frame to buffer, this happens when one of the other channels has failed or hung
+    return MFX_ERR_UNDEFINED_BEHAVIOR;
+}
+
+mfxStatus CBitstreamWriterForParallelEncoding::Reset() {
+    //we don't support reset for parallel encoding
+    return MFX_ERR_NOT_IMPLEMENTED;
+}
+
 CSmplBitstreamDuplicateWriter::CSmplBitstreamDuplicateWriter() : CSmplBitstreamWriter() {
     m_fSourceDuplicate = NULL;
     m_bJoined          = false;
@@ -765,14 +910,16 @@ mfxStatus CIVFFrameReader::ReadNextFrame(mfxBitstream* pBS) {
     READ_BYTES(&nBytesInFrame, sizeof(nBytesInFrame));
     CHECK_SET_EOS(pBS);
 
+    //check if bitstream has enough space to hold the frame
+    if (nBytesInFrame > pBS->MaxLength - pBS->DataLength - pBS->DataOffset) {
+        fseek(m_fSource, -long(sizeof(nBytesInFrame)), SEEK_CUR);
+        return MFX_ERR_NOT_ENOUGH_BUFFER;
+    }
+
     // read time stamp
     READ_BYTES(&nTimeStamp, sizeof(nTimeStamp));
     CHECK_SET_EOS(pBS);
 
-    //check if bitstream has enough space to hold the frame
-    if (nBytesInFrame > pBS->MaxLength - pBS->DataLength - pBS->DataOffset)
-        return MFX_ERR_NOT_ENOUGH_BUFFER;
-
     // read frame data
     READ_BYTES(pBS->Data + pBS->DataOffset + pBS->DataLength, nBytesInFrame);
     CHECK_SET_EOS(pBS);
@@ -857,6 +1004,8 @@ mfxStatus CIVFFrameWriter::WriteNextFrame(mfxBitstream* pMfxBitstream, bool isPr
         WriteFrameHeader();
 
         CSmplBitstreamWriter::WriteNextFrame(pMfxBitstream, isPrint);
+
+        m_frameNum++;
     }
 
     return MFX_ERR_NONE;
@@ -1556,6 +1705,57 @@ void QPFile::Reader::NextFrame() {
     ++m_CurFrameNum;
 }
 
+void TCBRCTestFile::Reader::ResetState(ReaderStatus set_sts) {
+    m_CurFrameNum = std::numeric_limits<mfxU32>::max();
+    m_ReaderSts   = set_sts;
+    m_FrameVals.clear();
+}
+
+mfxStatus TCBRCTestFile::Reader::Read(const msdk_string& strFileName, mfxU32 codecid) {
+    m_ReaderSts   = READER_ERR_NONE;
+    m_CurFrameNum = 0;
+
+    if (codecid != MFX_CODEC_AVC && codecid != MFX_CODEC_HEVC) {
+        ResetState(READER_ERR_CODEC_UNSUPPORTED);
+        return MFX_ERR_NOT_INITIALIZED;
+    }
+
+    std::ifstream ifs(strFileName, msdk_fstream::in);
+    if (!ifs.is_open()) {
+        ResetState(READER_ERR_FILE_NOT_OPEN);
+        return MFX_ERR_NOT_INITIALIZED;
+    }
+
+    FrameInfo frameInfo{};
+    std::string line;
+
+    mfxU32 n = 0;
+    while (TCBRCTestFile::get_line(ifs, line)) {
+        frameInfo.displayOrder    = TCBRCTestFile::ReadDisplayOrder(line);
+        frameInfo.targetFrameSize = TCBRCTestFile::ReadTargetFrameSize(line);
+        if (frameInfo.displayOrder == 0)
+            frameInfo.displayOrder = n;
+        m_FrameVals.push_back(frameInfo);
+        n++;
+    }
+    return MFX_ERR_NONE;
+}
+
+std::string TCBRCTestFile::Reader::GetErrorMessage() const {
+    return ReaderStatusToString(m_ReaderSts);
+}
+
+mfxU32 TCBRCTestFile::Reader::GetTargetFrameSize(mfxU32 displayOrder) const {
+    mfxU32 num = (mfxU32)m_FrameVals.size();
+    if (num == 0)
+        return 0;
+    for (mfxU32 i = 0; i < num - 1; i++) {
+        if (m_FrameVals.at(i + 1).displayOrder > displayOrder)
+            return m_FrameVals.at(i).targetFrameSize;
+    }
+    return m_FrameVals.at(num - 1).targetFrameSize;
+}
+
 mfxStatus ConvertFrameRate(mfxF64 dFrameRate, mfxU32* pnFrameRateExtN, mfxU32* pnFrameRateExtD) {
     MSDK_CHECK_POINTER(pnFrameRateExtN, MFX_ERR_NULL_PTR);
     MSDK_CHECK_POINTER(pnFrameRateExtD, MFX_ERR_NULL_PTR);
@@ -2346,6 +2546,9 @@ mfxStatus StrFormatToCodecFormatFourCC(msdk_char* strInput, mfxU32& codecFormat)
         else if ((0 == msdk_strcmp(strInput, MSDK_STRING("i422")))) {
             codecFormat = MFX_CODEC_I422;
         }
+        else if ((0 == msdk_strcmp(strInput, MSDK_STRING("p010")))) {
+            codecFormat = MFX_CODEC_P010;
+        }
         else
             sts = MFX_ERR_UNSUPPORTED;
     }
