@@ -9,6 +9,7 @@
 
 #include <stdarg.h>
 #include <memory>
+#include <regex>
 #include <string>
 #include "pipeline_encode.h"
 #include "pipeline_region_encode.h"
@@ -75,13 +76,25 @@ void PrintHelp(msdk_char* strAppName, const msdk_char* strErrorMessage, ...) {
 #if (defined(_WIN64) || defined(_WIN32))
     msdk_printf(MSDK_STRING(
         "   [-dual_gfx::<on,off,adaptive>] - prefer processing on both iGfx and dGfx simultaneously\n"));
+#endif
+#ifdef ONEVPL_EXPERIMENTAL
+    #if (defined(_WIN64) || defined(_WIN32))
+    msdk_printf(MSDK_STRING("   [-luid HighPart:LowPart] - setup adapter by LUID  \n"));
+    msdk_printf(MSDK_STRING("                                 For example: \"0x0:0x8a46\"  \n"));
+    #endif
+    msdk_printf(MSDK_STRING("   [-pci domain:bus:device.function] - setup device with PCI \n"));
+    msdk_printf(MSDK_STRING("                                 For example: \"0:3:0.0\"  \n"));
 #endif
     msdk_printf(MSDK_STRING(
-        "   [-dGfx] - preffer processing on dGfx (by default system decides), also can be set with index, for example: '-dGfx 1' \n"));
+        "   [-dGfx] - prefer processing on dGfx (by default system decides), also can be set with index, for example: '-dGfx 1' \n"));
     msdk_printf(
-        MSDK_STRING("   [-iGfx] - preffer processing on iGfx (by default system decides)\n"));
+        MSDK_STRING("   [-iGfx] - prefer processing on iGfx (by default system decides)\n"));
     msdk_printf(
-        MSDK_STRING("   [-AdapterNum] - specifies adpter number for processing, starts from 0\n"));
+        MSDK_STRING("   [-AdapterNum] - specifies adapter number for processing, starts from 0\n"));
+    msdk_printf(MSDK_STRING(
+        "   [-dispatcher:fullSearch]  - enable search for all available implementations in oneVPL dispatcher\n"));
+    msdk_printf(MSDK_STRING(
+        "   [-dispatcher:lowLatency]  - enable limited implementation search and query in oneVPL dispatcher\n"));
 #ifdef MOD_ENC
     MOD_ENC_PRINT_HELP;
 #endif
@@ -176,7 +189,7 @@ void PrintHelp(msdk_char* strAppName, const msdk_char* strErrorMessage, ...) {
     msdk_printf(MSDK_STRING(
         "   [-lowpower:<on,off>]     - Turn this option ON to enable QuickSync Fixed Function (low-power HW) encoding mode\n"));
     msdk_printf(MSDK_STRING(
-        "   [-ir_type]               - Intra refresh type. 0 - no refresh, 1 - vertical refresh, 2 - horisontal refresh, 3 - slice refresh\n"));
+        "   [-ir_type]               - Intra refresh type. 0 - no refresh, 1 - vertical refresh, 2 - horizontal refresh, 3 - slice refresh\n"));
     msdk_printf(MSDK_STRING(
         "   [-ir_cycle_size]         - Number of pictures within refresh cycle starting from 2\n"));
     msdk_printf(MSDK_STRING(
@@ -188,20 +201,20 @@ void PrintHelp(msdk_char* strAppName, const msdk_char* strErrorMessage, ...) {
     msdk_printf(
         MSDK_STRING("   [-TransformSkip:<on,off>]- Turn this option ON to enable transformskip\n"));
     msdk_printf(MSDK_STRING(
-        "  -AvcTemporalLayers [array:Layer.Scale]    Configures the temporal layers hierarchy"
+        "   [-AvcTemporalLayers <array:Layer.Scale>]    Configures the temporal layers hierarchy"
         "                                            example: \"3 1 2 4 0 0 0 0 0\" - # 3 temporal layers(temporal_id = 0,1,2)\n"));
     msdk_printf(MSDK_STRING(
-        "  -BaseLayerPID <pid>                       Sets priority ID for the base layer\n"));
+        "   [-BaseLayerPID <pid>]                       Sets priority ID for the base layer\n"));
     msdk_printf(MSDK_STRING(
-        "  -SPSId <pid>                              Sets sequence parameter set ID\n"));
-    msdk_printf(
-        MSDK_STRING("  -PPSId <pid>                              Sets picture parameter set ID\n"));
+        "   [-SPSId <pid>]                              Sets sequence parameter set ID\n"));
     msdk_printf(MSDK_STRING(
-        "  -PicTimingSEI:<on,off>                    Enables or disables picture timing SEI\n"));
+        "   [-PPSId <pid>]                              Sets picture parameter set ID\n"));
     msdk_printf(MSDK_STRING(
-        "  -NalHrdConformance:<on,off>               Enables or disables picture HRD conformance\n"));
+        "   [-PicTimingSEI:<on,off>]                    Enables or disables picture timing SEI\n"));
     msdk_printf(MSDK_STRING(
-        "  -VuiNalHrdParameters:<on,off>             Enables or disables NAL HRD parameters in VUI header\n"));
+        "   [-NalHrdConformance:<on,off>]               Enables or disables picture HRD conformance\n"));
+    msdk_printf(MSDK_STRING(
+        "   [-VuiNalHrdParameters:<on,off>]             Enables or disables NAL HRD parameters in VUI header\n"));
     msdk_printf(MSDK_STRING(
         "   [-ppyr:<on,off>]         - Turn this option ON to enable P-pyramid (by default the decision is made by library)\n"));
     msdk_printf(MSDK_STRING(
@@ -267,6 +280,10 @@ void PrintHelp(msdk_char* strAppName, const msdk_char* strErrorMessage, ...) {
         "   [-dump fileName]         - dump MSDK components configuration to the file in text form\n"));
     msdk_printf(MSDK_STRING(
         "   [-qpfile <filepath>]     - if specified, the encoder will take frame parameters (frame number, QP, frame type) from text file\n"));
+    msdk_printf(MSDK_STRING(
+        "   [-tcbrctestfile <filepath>] - if specified, the encoder will take targetFrameSize parameters for TCBRC test from text file. \
+                            The parameters for TCBRC should be calculated based on the channel width conditions, sample doesn't have this functionality. \
+                            Therefore the file data from <filepath> is used for TCBRC test. This is a test model\n"));
     msdk_printf(MSDK_STRING(
         "   [-usei]                  - insert user data unregistered SEI. eg: 7fc92488825d11e7bb31be2e44b06b34:0:MSDK (uuid:type<0-preifx/1-suffix>:message)\n"));
     msdk_printf(MSDK_STRING(
@@ -311,16 +328,17 @@ void PrintHelp(msdk_char* strAppName, const msdk_char* strErrorMessage, ...) {
     msdk_printf(MSDK_STRING(
         "   [-et:brc:<on,off>]       - flag for enabling functionality: QP calculation for frame encoding, encoding status calculation after frame encoding\n"));
     msdk_printf(MSDK_STRING(
-        "   [-ScenarioInfo n] - Sets scenario info. 0=unknown, 7=MFX_SCENARIO_GAME_STREAMING, 8=MFX_SCENARIO_REMOTE_GAMING \n"));
+        "   [-ScenarioInfo n]        - Sets scenario info. 0=unknown, 7=MFX_SCENARIO_GAME_STREAMING, 8=MFX_SCENARIO_REMOTE_GAMING \n"));
     msdk_printf(
         MSDK_STRING("   [-ExtBrcAdaptiveLTR:<on,off>] - Set AdaptiveLTR for implicit extbrc\n"));
-    msdk_printf(
-        MSDK_STRING(
-            "Example: %s h265 -i InputYUVFile -o OutputEncodedFile -w width -h height -hw -p 2fca99749fdb49aeb121a5b63ef568f7\n"),
-        strAppName);
     msdk_printf(MSDK_STRING(
         "   [-preset <default,dss,conference,gaming>] - Use particular preset for encoding parameters\n"));
     msdk_printf(MSDK_STRING("   [-pp] - Print preset parameters\n"));
+    msdk_printf(MSDK_STRING("   [-ivf:<on,off>] - Turn IVF header on/off\n"));
+    msdk_printf(MSDK_STRING(
+        "   [-api_ver_init::<1x,2x>]  - select the api version for the session initialization\n"));
+    msdk_printf(MSDK_STRING("   [-rbf] - read frame-by-frame from the input (sw lib only)\n"));
+
 #if D3D_SURFACES_SUPPORT
     msdk_printf(MSDK_STRING("   [-d3d] - work with d3d surfaces\n"));
     msdk_printf(MSDK_STRING("   [-d3d11] - work with d3d11 surfaces\n"));
@@ -373,7 +391,7 @@ void PrintHelp(msdk_char* strAppName, const msdk_char* strErrorMessage, ...) {
     msdk_printf(MSDK_STRING("User module options: \n"));
     msdk_printf(MSDK_STRING(
         "   [-angle 180] - enables 180 degrees picture rotation before encoding, CPU implementation by default. Rotation requires NV12 input. Options -tff|bff, -dstw, -dsth, -d3d are not effective together with this one, -nv12 is required.\n"));
-    msdk_printf(MSDK_STRING("   [-opencl] - rotation implementation through OPENCL\n"));
+    msdk_printf(MSDK_STRING("   [-opencl] - rotation implementation through OPENCL\n\n"));
     msdk_printf(
         MSDK_STRING(
             "Example: %s h264|h265|mpeg2|mvc|jpeg -i InputYUVFile -o OutputEncodedFile -w width -h height -angle 180 -opencl \n"),
@@ -407,6 +425,13 @@ mfxStatus ParseAdditionalParams(msdk_char* strInput[],
         i += 1;
     }
 
+    else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-tcbrctestfile"))) {
+        VAL_CHECK(i + 1 >= nArgNum, i, strInput[i]);
+        pParams->TCBRCFileMode    = true;
+        pParams->strTCBRCFilePath = strInput[++i];
+        i += 1;
+    }
+
     else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-BaseLayerPID"))) {
         VAL_CHECK(i + 1 >= nArgNum, i, strInput[i]);
         if (MFX_ERR_NONE != msdk_opt_read(strInput[++i], pParams->nBaseLayerPID)) {
@@ -646,6 +671,92 @@ mfxStatus ParseAdditionalParams(msdk_char* strInput[],
     else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-TransformSkip:off"))) {
         pParams->nTransformSkip = MFX_CODINGOPTION_OFF;
     }
+    else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-ivf:on"))) {
+        pParams->nIVFHeader = MFX_CODINGOPTION_ON;
+    }
+    else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-ivf:off"))) {
+        pParams->nIVFHeader = MFX_CODINGOPTION_OFF;
+    }
+    else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-dispatcher:fullSearch"))) {
+        pParams->dispFullSearch = true;
+    }
+    else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-dispatcher:lowLatency"))) {
+        pParams->dispFullSearch = false;
+    }
+    else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-api_ver_init::1x"))) {
+        pParams->verSessionInit = API_1X;
+    }
+    else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-api_ver_init::2x"))) {
+        pParams->verSessionInit = API_2X;
+    }
+    else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-rbf"))) {
+        pParams->bReadByFrame = true;
+    }
+#ifdef ONEVPL_EXPERIMENTAL
+    else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-pci"))) {
+        msdk_char deviceInfo[MSDK_MAX_FILENAME_LEN];
+        VAL_CHECK(i + 1 >= nArgNum, i, strInput[i]);
+        i++;
+        if ((msdk_strlen(strInput[i]) + 1) > MSDK_ARRAY_LEN(deviceInfo)) {
+            PrintHelp(strInput[0], MSDK_STRING("error: '-pci' arguments is too long\n"));
+            return MFX_ERR_UNSUPPORTED;
+        }
+        if (MFX_ERR_NONE != msdk_opt_read(strInput[i], deviceInfo)) {
+            PrintHelp(strInput[0], MSDK_STRING("error: '-pci' arguments is invalid\n"));
+            return MFX_ERR_UNSUPPORTED;
+        }
+
+        // template: <domain:bus:device.function>
+        std::string temp = std::string(deviceInfo);
+        const std::regex pieces_regex("([0-9]+):([0-9]+):([0-9]+).([0-9]+)");
+        std::smatch pieces_match;
+
+        // pieces_match = [full match, PCIDomain, PCIBus, PCIDevice, PCIFunction]
+        if (std::regex_match(temp, pieces_match, pieces_regex) && pieces_match.size() == 5) {
+            pParams->PCIDomain      = std::atoi(pieces_match[1].str().c_str());
+            pParams->PCIBus         = std::atoi(pieces_match[2].str().c_str());
+            pParams->PCIDevice      = std::atoi(pieces_match[3].str().c_str());
+            pParams->PCIFunction    = std::atoi(pieces_match[4].str().c_str());
+            pParams->PCIDeviceSetup = true;
+        }
+        else {
+            PrintHelp(
+                strInput[0],
+                MSDK_STRING(
+                    "error: format of '-pci' arguments is invalid, please, use: domain:bus:device.function \n"));
+            return MFX_ERR_UNSUPPORTED;
+        }
+    }
+    #if defined(_WIN32)
+    else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-luid"))) {
+        // <HighPart:LowPart>
+        msdk_char luid[MSDK_MAX_FILENAME_LEN];
+        VAL_CHECK(i + 1 >= nArgNum, i, strInput[i]);
+        i++;
+        if (MFX_ERR_NONE != msdk_opt_read(strInput[i], luid)) {
+            PrintHelp(strInput[0], MSDK_STRING("error: '-luid' arguments is invalid\n"));
+            return MFX_ERR_UNSUPPORTED;
+        }
+
+        std::string temp = std::string(luid);
+        const std::regex pieces_regex("(0[xX][0-9a-fA-F]+):(0[xX][0-9a-fA-F]+)");
+        std::smatch pieces_match;
+
+        // pieces_match = [full match, HighPart, LowPart]
+        if (std::regex_match(temp, pieces_match, pieces_regex) && pieces_match.size() == 3) {
+            pParams->luid.HighPart = std::strtol(pieces_match[1].str().c_str(), 0, 16);
+            pParams->luid.LowPart  = std::strtol(pieces_match[2].str().c_str(), 0, 16);
+        }
+        else {
+            PrintHelp(
+                strInput[0],
+                MSDK_STRING(
+                    "error: format of '-luid' arguments is invalid, please, use: HighPart:LowPart \n"));
+            return MFX_ERR_UNSUPPORTED;
+        }
+    }
+    #endif
+#endif
     else {
         return MFX_ERR_NOT_FOUND;
     }
@@ -670,10 +781,12 @@ mfxStatus ParseInputString(msdk_char* strInput[], mfxU8 nArgNum, sInputParams* p
     pParams->EncodeFourCC       = 0;
     pParams->nPRefType          = MFX_P_REF_DEFAULT;
     pParams->QPFileMode         = false;
+    pParams->TCBRCFileMode      = false;
     pParams->BitrateLimit       = MFX_CODINGOPTION_OFF;
     pParams->adapterType        = mfxMediaAdapterType::MFX_MEDIA_UNKNOWN;
     pParams->dGfxIdx            = -1;
     pParams->adapterNum         = -1;
+    pParams->dispFullSearch     = DEF_DISP_FULLSEARCH;
     pParams->RoundingOffsetFile = NULL;
 #if defined(ENABLE_V4L2_SUPPORT)
     pParams->MipiPort   = -1;
@@ -725,6 +838,11 @@ mfxStatus ParseInputString(msdk_char* strInput[], mfxU8 nArgNum, sInputParams* p
             }
             VAL_CHECK(i + 1 >= nArgNum, i, strInput[i]);
             pParams->strDevicePath = strInput[++i];
+
+            size_t pos = pParams->strDevicePath.find("renderD");
+            if (pos != std::string::npos) {
+                pParams->DRMRenderNodeNum = std::stoi(pParams->strDevicePath.substr(pos + 7, 3));
+            }
         }
 #endif
         else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-dGfx"))) {
@@ -735,9 +853,15 @@ mfxStatus ParseInputString(msdk_char* strInput[], mfxU8 nArgNum, sInputParams* p
                     return MFX_ERR_UNSUPPORTED;
                 }
             }
+#if (defined(_WIN64) || defined(_WIN32)) && (MFX_VERSION >= 1031)
+            pParams->bPreferdGfx = true;
+#endif
         }
         else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-iGfx"))) {
             pParams->adapterType = mfxMediaAdapterType::MFX_MEDIA_INTEGRATED;
+#if (defined(_WIN64) || defined(_WIN32)) && (MFX_VERSION >= 1031)
+            pParams->bPreferiGfx = true;
+#endif
         }
         else if (0 == msdk_strcmp(strInput[i], MSDK_STRING("-AdapterNum"))) {
             VAL_CHECK(i + 1 >= nArgNum, i, strInput[i]);
@@ -1634,7 +1758,7 @@ mfxStatus ParseInputString(msdk_char* strInput[], mfxU8 nArgNum, sInputParams* p
 
     // Ignoring user-defined Async Depth for LA
     if (pParams->nAsyncDepth == 0 || pParams->nMaxSliceSize) {
-        pParams->nAsyncDepth = 1;
+        pParams->nAsyncDepth = 4;
     }
 
     if (pParams->UseRegionEncode) {
@@ -1660,6 +1784,50 @@ mfxStatus ParseInputString(msdk_char* strInput[], mfxU8 nArgNum, sInputParams* p
         msdk_printf(MSDK_STRING("Warning: async depth changed to 1 for partial output"));
     }
 
+#if (defined(_WIN64) || defined(_WIN32)) && (MFX_VERSION >= 1031)
+    if (pParams->bPreferdGfx && pParams->bPreferiGfx) {
+        msdk_printf(MSDK_STRING("Warning: both dGfx and iGfx flags set. iGfx will be preferred"));
+        pParams->bPreferdGfx = false;
+    }
+#endif
+
+    mfxU16 mfxU16Limit = std::numeric_limits<mfxU16>::max();
+    if (pParams->MaxKbps > mfxU16Limit || pParams->nBitRate > mfxU16Limit ||
+        pParams->InitialDelayInKB > mfxU16Limit || pParams->BufferSizeInKB > mfxU16Limit) {
+        mfxU32 maxVal               = std::max<mfxU32>({ pParams->MaxKbps,
+                                                         pParams->nBitRate,
+                                                         pParams->InitialDelayInKB,
+                                                         pParams->BufferSizeInKB });
+        pParams->nBitRateMultiplier = (mfxU16)std::ceil(static_cast<double>(maxVal) / mfxU16Limit);
+        msdk_printf(MSDK_STRING("WARNING: BitRateMultiplier(-bm) was updated, new value - %d. \n"),
+                    pParams->nBitRateMultiplier);
+
+        auto recalculate = [mfxU16Limit, pParams](mfxU32& param, std::string paramName) {
+            if (param) {
+                if (param > mfxU16Limit) {
+                    msdk_printf(MSDK_STRING("WARNING: %s (%d) > allow limit (%d). \n"),
+                                paramName.c_str(),
+                                param,
+                                mfxU16Limit);
+                }
+                param = param / pParams->nBitRateMultiplier;
+                msdk_printf(MSDK_STRING("WARNING: %s was updated, new value: %d. \n"),
+                            paramName.c_str(),
+                            param);
+            }
+        };
+
+        recalculate(pParams->MaxKbps, "MaxKbps");
+        recalculate(pParams->nBitRate, "nBitRate(-b)");
+        recalculate(pParams->InitialDelayInKB, "InitialDelayInKB");
+        recalculate(pParams->BufferSizeInKB, "BufferSizeInKB");
+    }
+
+    if (pParams->bUseHWLib && pParams->bReadByFrame) {
+        PrintHelp(strInput[0], MSDK_STRING("-rbf (Read by frame) is not supported in hw lib"));
+        return MFX_ERR_UNSUPPORTED;
+    }
+
     return MFX_ERR_NONE;
 }
 
